@@ -1,11 +1,24 @@
 package Barcode::DataMatrix::Engine;
 
+=head1 Barcode::DataMatrix::Engine
+
+The engine which generates the data matrix bitmap.
+
+=cut
+
 use strict;
 no warnings qw(uninitialized);
 use Barcode::DataMatrix::Reed;
 use Barcode::DataMatrix::Constants ();
 use Barcode::DataMatrix::CharDataFiller ();
 use Data::Dumper;$Data::Dumper::Useqq = 1;
+
+=head2 DEBUG
+
+Turn on/off general debugging information.
+
+=cut
+
 use constant DEBUG => 0;
 
 our %DEBUG = (
@@ -23,24 +36,84 @@ our (@GFI,@GFL,%POLY,@FORMATS,@C1);
 *FORMATS = \@Barcode::DataMatrix::Constants::FORMATS;
 *C1      = \@Barcode::DataMatrix::Constants::C1;
 
-sub E_ASCII  () { 0 }
-sub E_C40    () { 1 }
-sub E_TEXT   () { 2 }
-sub E_BASE256() { 3 }
-sub E_NONE   () { 4 }
-sub E_AUTO   () { 5 }
+=head2 E_ASCII
+
+Represent the ASCII encoding type.
+
+=cut
+
+sub E_ASCII { 0 }
+
+=head2 E_C40
+
+Represent the C40 encoding type.  (upper case alphanumeric)
+
+=cut
+
+sub E_C40 { 1 }
+
+=head2 E_TEXT
+
+Represent the TEXT encoding type.  (lower case alphanumeric)
+
+=cut
+
+sub E_TEXT { 2 }
+
+=head2 E_BASE256
+
+Represent the BASE256 encoding type.
+
+=cut
+
+sub E_BASE256 { 3 }
+
+=head2 E_NONE
+
+Represent the when there is no encoding type.
+
+=cut
+
+sub E_NONE { 4 }
+
+=head2 E_AUTO
+
+Represent the when the encoding type is automatically set.
+
+=cut
+
+sub E_AUTO { 5 }
 
 our $N = 255;
 
+=head2 Types
+
+Return a list of encoding types.
+
+=cut
+
 sub Types {
 	return qw( ASCII C40 TEXT BASE256 NONE AUTO );
 }
 
-sub stringToType($) {
+=head2 stringToType (type_name)
+
+Return the integer representing the type from the type name.
+
+=cut
+
+sub stringToType {
 	my $m = 'E_'.shift;
 	return eval { __PACKAGE__->$m(); };
 }
-sub typeToString($) {
+
+=head2 typeToString (type_integer)
+
+Return the type name from the integer representing the type.
+
+=cut
+
+sub typeToString {
 	my $i = shift;
 	for (Types) {
 		return $_ if stringToType($_) == $i and defined $i;
@@ -50,7 +123,13 @@ sub typeToString($) {
 
 our @encName = map { typeToString $_ } 0..5;
 
-sub stringToFormat($) {
+=head2 stringToFormat (format_string)
+
+Convert a "width x height" format string into an internal format specification.
+
+=cut
+
+sub stringToFormat {
 	my $sz = shift;
 	return unless $sz;
 	my ($w,$h) = map { +int } split /\s*x\s*/,$sz,2;
@@ -62,6 +141,12 @@ sub stringToFormat($) {
 	return $r;
 }
 
+=head2 setType (type_name)
+
+Set the encoding type from the given type name.
+
+=cut
+
 sub setType {
 	my $self = shift;
 	my $type = shift;
@@ -74,6 +159,12 @@ sub setType {
 	return;
 }
 
+=head2 new
+
+Construct a C<Barcode::DataMatrix::Engine> object.
+
+=cut
+
 sub new {
 	my $self = bless{},shift;
 	$self->init();
@@ -90,6 +181,12 @@ sub new {
 	return $self;
 }
 
+=head2 init
+
+Initialize some of the basic C<Barcode::DataMatrix::Engine> data.
+
+=cut
+
 sub init {
 	my $self = shift;
 	my %p = (
@@ -104,6 +201,13 @@ sub init {
 	}
 }
 
+=head2 ProcessTilde
+
+Handle special or control characters, which are prefixed by a tilde C<~>
+when encoding.
+
+=cut
+
 sub ProcessTilde {
 	my $self = shift;
 	my $s = $self->{code};
@@ -119,33 +223,39 @@ sub ProcessTilde {
 		s{^~3}{ $as->[0] = ''; "\352" }e;
 		s{^~5}{ $as->[0] = ''; "\354" }e;
 		s{^~6}{ $as->[0] = ''; "\355" }e;
-			s{~7(.{6})}{do{
-				my $d = int $1;
-				#warn "There is $d got from $1\n";
-				if ($d < 127) {
-					$d = chr($d+1);
-				}
-				elsif($d < 16383) {
-					$d =
-						chr( ( $d - 127 ) / 254 + 128 ).
-						chr( ( $d - 127 ) % 254 + 1 );
-				}
-				else{
-					$d =
-						chr( int( ( $d - 16383 ) / 64516 + 192 ) ).
-						chr( int( ( $d - 16383 ) / 254 ) % 254 + 1 ).
-						chr( int( ( $d - 16383 ) % 254 + 1 ) );
-				}
-				$as->[$-[0]] = $d;
-				warn "PT affect as[$-[0]] = ".join('+', map ord, split //, $d) if $DEBUG{TRACE};
-				"\361"
-			}}ge;
+		s{~7(.{6})}{do{
+			my $d = int $1;
+			#warn "There is $d got from $1\n";
+			if ($d < 127) {
+				$d = chr($d+1);
+			}
+			elsif($d < 16383) {
+				$d =
+					chr( ( $d - 127 ) / 254 + 128 ).
+					chr( ( $d - 127 ) % 254 + 1 );
+			}
+			else{
+				$d =
+					chr( int( ( $d - 16383 ) / 64516 + 192 ) ).
+					chr( int( ( $d - 16383 ) / 254 ) % 254 + 1 ).
+					chr( int( ( $d - 16383 ) % 254 + 1 ) );
+			}
+			$as->[$-[0]] = $d;
+			warn "PT affect as[$-[0]] = ".join('+', map ord, split //, $d) if $DEBUG{TRACE};
+			"\361"
+		}}ge;
 		s{~(.)}{$1 eq '~' ? '~' : $1}ge;
 		warn "[C9] ProcessTilde($self->{code}) => ".Dumper($_) if $DEBUG{TRACE};
 		return $self->{code} = $_;
 	}
 }
 
+=head2 CalcReed (ai, err)
+
+Return the message as a Reed-Solomon encoded array.
+
+=cut
+
 sub CalcReed { # (int ai[], int i, int j) : void
 	my ($ai,$err) = @_;
 	my $rv = Barcode::DataMatrix::Reed::encode($ai,$err);
@@ -160,7 +270,7 @@ sub CalcReed { # (int ai[], int i, int j) : void
 #		return $GFI[$k];
 #	}
 #	sub short($) { $_[0] & 0xFF; }
-#		
+#
 #	my ($ai,$j) = @_;
 #	my $i = @$ai;
 #	warn "CalcReed(ai {".join(" ",grep{+defined}@$ai)."},$i,$j)\n" if $DEBUG{CALC};
@@ -177,14 +287,44 @@ sub CalcReed { # (int ai[], int i, int j) : void
 #    return $ai;
 }
 
-sub A253($$) # C8 (int i, int j) : int 
+=head2 A253 (i, j)
+
+Return padding codewords via the 253-state algorithm.
+
+For more information see
+L<http://grandzebu.net/informatique/codbar-en/datamatrix.htm>.
+
+The relevant text for this algorithm is reproduced here.
+
+If the symbol is not full, pad C<CW>s are required. After the last data
+C<CW>, the 254 C<CW> indicates the end of the datas or the return to ASCII
+method. First padding C<CW> is 129 and next padding C<CW>s are computed with
+the 253-state algorithm.
+
+=head3 The 253-state algorithm
+
+Let C<P> be the number of data C<CW>s from the beginning of the data, C<R> a
+pseudo random number and C<CW> the required pad C<CW>.
+
+    R = ((149 * P) MOD 253) + 1
+    CW = (129 + R) MOD 254
+
+=cut
+
+sub A253 # C8 (int i, int j) : int
 {
 	my ($i,$j) = @_;
     my $l = $i + (149 * $j) % 253 + 1;
     return $l <= 254 ? $l : $l - 254;
 }
 
-sub CreateBitmap() #CB (int ai[], String as[]) : int[][]
+=head2 CreateBitmap
+
+Generate and return the bitmap representing the message.
+
+=cut
+
+sub CreateBitmap #CB (int ai[], String as[]) : int[][]
 {
 	my $self = shift;
 	my ($ai,$as) = @$self{qw(ai as)};
@@ -231,17 +371,17 @@ sub CreateBitmap() #CB (int ai[], String as[]) : int[][]
     return if $k == 30;
     my $l = $k;
 	@$self{qw(
-		rows      
-		cols      
-		datarows  
-		datacols  
-		regions   
-		maprows   
-		mapcols   
-		totaldata 
-		totalerr  
-		reeddata  
-		reederr   
+		rows
+		cols
+		datarows
+		datacols
+		regions
+		maprows
+		mapcols
+		totaldata
+		totalerr
+		reeddata
+		reederr
 		reedblocks
 	)} = @{$FORMATS[$l]}[0..11];
 	DEBUG and print "Format: $self->{rows}x$self->{cols}; Data: $self->{totaldata}; i=$i; blocks = $self->{reedblocks}\n";
@@ -261,6 +401,13 @@ sub CreateBitmap() #CB (int ai[], String as[]) : int[][]
     return $self->{bitmap} = $self->GenData($self->ecc($l,$ai1));
 }
 
+=head2 ecc (format, ai)
+
+Return the ECC200 (DataMatrix) array, formatted for the appropriate matrix
+size.
+
+=cut
+
 sub ecc {
 	my $self = shift;
 	my $format = shift;
@@ -282,7 +429,7 @@ sub ecc {
 			#warn "144 fix: decrease block $_ to size 155 from @{[ 0+@{$blocks[$_]} ]}";
         	$#{$blocks[$_]} -= 1;
         }
-		
+
 		CalcReed($blocks[$_], $err);
 	}
     warn "Calc reed=\n".
@@ -301,22 +448,55 @@ sub ecc {
 	return \@rv;
 }
 
+=head2 isCDigit (character)
+
+Return true if the character is a digit.
+
+=cut
+
 sub isCDigit { # C1*
 	return shift =~ /^[0-9]$/ ? 1 : 0;
 }
+
+=head2 isIDigit (character_code)
+
+Return true if the character code represents a digit.
+
+=cut
+
 sub isIDigit { # C1
 	my $i = shift;
 	return ( $i >= 48 && $i <= 57 ) ? 1 : 0;
 }
+
+=head2 isILower (character_code)
+
+Return true if the character code represents a lower case letter.
+
+=cut
+
 sub isILower {
 	my $i = shift;
 	return ( $i >= ord('a') && $i <= ord('z') ) ? 1 : 0;
 }
+
+=head2 isIUpper (character_code)
+
+Return true if the character code represents an upper case letter.
+
+=cut
+
 sub isIUpper {
 	my $i = shift;
 	return ( $i >= ord('A') && $i <= ord('Z') ) ? 1 : 0;
 }
 
+=head2 DetectEncoding
+
+Detect the encoding type.
+
+=cut
+
 sub DetectEncoding() #C4 (int i, int ai[], int ai1[], String as[]) : int
 {
 	my $self = shift;
@@ -342,7 +522,7 @@ sub DetectEncoding() #C4 (int i, int ai[], int ai1[], String as[]) : int
 			warn("DetectENC: while at $iterator ce=$encName[$self->{currentEncoding}] k1=$encName[$k1] l2=$encName[$l2]\n") if $DEBUG{EAUTO};
             my $flag1 = 0;
             if(
-            	$iterator + 1 < $i 
+            	$iterator + 1 < $i
             	and isIDigit($ai->[$iterator])
             	and isIDigit($ai->[$iterator + 1])
             ){
@@ -419,8 +599,13 @@ sub DetectEncoding() #C4 (int i, int ai[], int ai1[], String as[]) : int
     return $j1;
 }
 
+=head2 EncodeASCII (i, ai, ai1, as)
 
-sub EncodeASCII { #CE (int i; int ai[], int ai1[], String as[]) : int 
+Encode the message as ASCII.
+
+=cut
+
+sub EncodeASCII { #CE (int i; int ai[], int ai1[], String as[]) : int
 	my $self = shift;
 	warn "[CE] EncodeASCII(@_)\n" if $DEBUG{TRACE};
 	my ($i,$ai,$ai1,$as) = @_;
@@ -429,8 +614,8 @@ sub EncodeASCII { #CE (int i; int ai[], int ai1[], String as[]) : int
     my $flag = 0;
     for(my $k = 0; $k < $i; $k++) {
         my $flag1 = 0;
-        if(	
-        	$k < $i - 1 
+        if(
+        	$k < $i - 1
         	and isIDigit($ai->[$k])
         	and isIDigit($ai->[$k+1])
         ) {
@@ -477,6 +662,12 @@ sub EncodeASCII { #CE (int i; int ai[], int ai1[], String as[]) : int
     return $j;
 }
 
+=head2 SelectEncoding (j, ai, i)
+
+Select a new encoding type for the message.
+
+=cut
+
 sub SelectEncoding #C3 (int ai[], int i, int j, String as[]) : int # DefineEncoding??
                    #iterator, ai, encoding
 {
@@ -484,15 +675,15 @@ sub SelectEncoding #C3 (int ai[], int i, int j, String as[]) : int # DefineEncod
 	#(ai,i: encoding,j: iterator,as)
 	my $self = shift;
 	warn "[C3] SelectEncoding(@_)\n" if $DEBUG{TRACE};
-	
+
 	my $j = shift;
-	
+
 	my $ai = shift;
 	$ai = $self->{ai} unless defined $ai;
 
 	my $i = shift || $self->{currentEncoding};
 	$i = $self->{currentEncoding} unless defined $i;
-	
+
 	my $as = $self->{as};
     my $d  = 0.0;
     my $d2 = 1.0;
@@ -514,24 +705,24 @@ sub SelectEncoding #C3 (int ai[], int i, int j, String as[]) : int # DefineEncod
     	warn "SelectEncoding: have as[$j]: $as->[$j]\n" if defined $as->[$j] and $DEBUG{EAUTO};
         my $c = $ai->[$j];
         return E_ASCII if defined $as->[$j];
-        
+
         if    ( isIDigit($c) )        { $d += 0.5 }
         elsif ( $c > 127 )            { $d = int( $d + 0.5 ) + 2; }
         else                          { $d = int( $d + 0.5 ) + 1; }
-        
+
         if    ( @{ $C1[$c] } == 1 )   { $d2 += 0.66000000000000003; }
         elsif ( $c > 127 )            { $d2 += 2.6600000000000001;  }
         else                          { $d2 += 1.3300000000000001;  }
         my $c1 = $c;
         if( isIUpper($c) )            { $c1 = ord lc chr $c; }
         if( isILower($c) )            { $c1 = ord uc chr $c; }
-        
+
         if    ( @{ $C1[$c1] } == 1)   { $d3 += 0.66000000000000003; }
         elsif ( $c1 > 127 )           { $d3 += 2.6600000000000001;  }
         else                          { $d3 += 1.3300000000000001;  }
 
         $d4++;
-        
+
         if($j - $k >= 4) {
         	#warn "$j-$k >= 4: $d $d2 $d3 $d4\n";
             return E_ASCII   if $d  + 1.0 <= $d2 and $d + 1.0 <= $d3 and $d + 1.0 <= $d4;
@@ -552,6 +743,12 @@ sub SelectEncoding #C3 (int ai[], int i, int j, String as[]) : int # DefineEncod
     return E_C40;
 }
 
+=head2 EncodeC40TEXT (i, ai, ai1, ai2, flag, flag1, flag2)
+
+Encode the message as C40/TEXT.
+
+=cut
+
 sub EncodeC40TEXT { # C6 #(int i, int ai[], int ai1[], int ai2[], boolean flag, boolean flag1, boolean flag2) : int
 	#warn "[C6] EncodeC40TEXT\n";
 	my $self = shift;
@@ -613,33 +810,60 @@ sub EncodeC40TEXT { # C6 #(int i, int ai[], int ai1[], int ai2[], boolean flag,
     return $j;
 }
 
+=head2 state255 (V, P)
 
-sub state255($$) # (int V, int P) : int
+The 255-state algorithm.  Used when encoding strings with the BASE256 type.
+
+This information originally from
+L<http://grandzebu.net/informatique/codbar-en/datamatrix.htm>.
+
+Let C<P> the number of data C<CW>s from the beginning of datas (C<CW> = code
+word).  Let C<R> be a pseudo random number, C<V> the base 256 C<CW> value
+and C<CW> the required C<CW>.
+
+    R = ((149 * P) MOD 255) + 1
+    CW = (V + R) MOD 256
+
+=cut
+
+sub state255 # (int V, int P) : int
 {
-	#The 255-state algorithm.
-	#Let P the number of data CWs from the beginning of datas,
-	#R a pseudo random number,
-	#V the base 256 CW value and CW the required CW.
-	#R = ((149 * P) MOD 255) + 1
-	#CW = (V + R) MOD 256
     my ($V,$P) = @_;
     return ( $V + (149 * $P) % 255 + 1 ) % 256;
 }
 
+=head2 hexary (src)
+
+Return a string representation of the input hexadecimal number.
+
+=cut
+
 sub hexary {
 	join(" ",map{ sprintf '%02x',$_} @{ shift() } )
 }
 
+=head2 decary (src)
+
+Return a string representation of the input decimal number.
+
+=cut
+
 sub decary {
 	join(" ",map{ sprintf '%3d',$_} @{ shift() } )
 }
 
+=head2 EncodeBASE256 (i, hint, src, stat, res, flag)
+
+Encode the message as BASE256.
+
+=cut
+
 sub EncodeBASE256 {
 	my $self = shift;
 	my ($i,$hint,$src,$stat,$res,$flag) = @_;
     my $j = 0;
     my $xv = [];
-    my $k = 
+    my $k =
     my $l = $stat->[0];
     my $flag1 = 0;
     my $j1 = 0;
@@ -665,12 +889,18 @@ sub EncodeBASE256 {
     return $l;
 }
 
+=head2 GenData (ai)
+
+Generate and return the data for the DataMatrix bitmap from the input array.
+
+=cut
+
 sub GenData { # CC (int ai[]) : int[][]
     my $self = shift;
     my ($ai) = @_;
     warn "[CC] GenData: ".join(",",@$ai)." [$self->{rows} x $self->{cols} : $self->{regions} : $self->{datacols}x$self->{datarows}]\n" if $DEBUG{TRACE};
 	my $ai1 = [ map { [ (undef) x $self->{rows} ] } 1..$self->{cols} ]; # reverse cols/rows here, for correct access ->[][]
-	
+
     my $i = my $j = 0;
     # Draw border
     if($self->{regions} == 2) {
@@ -722,6 +952,12 @@ sub GenData { # CC (int ai[]) : int[][]
     return $ai1;
 }
 
+=head2 FillBorder (ai, i, j, k, l)
+
+Fill the border of the ECC200 data matrix bitmap.
+
+=cut
+
 sub FillBorder { # CD (int ai[][], int i, int j, int k, int l) : void
 	my ($ai,$i,$j,$k,$l) = @_;
 	#warn "[CD] FillBorder([".join(",",@$ai)."],$i,$j,$k,$l)\n";
@@ -739,6 +975,12 @@ sub FillBorder { # CD (int ai[][], int i, int j, int k, int l) : void
     }
 }
 
+=head2 FillCharData (ncol, nrow, array)
+
+Fill the data matrix with the character data in the given message array.
+
+=cut
+
 sub FillCharData { # (int ncol; int nrow; int array;) : void
 	my ($ncol,$nrow,$array) = @_;
 	Barcode::DataMatrix::CharDataFiller->new($ncol,$nrow,$array);
